#include "OkanMos.h"
#include <iostream>
using namespace std;

OkanMos::OkanMos()
{
	image[0] = NULL;
	image[1] = NULL;
//	inputMask[0] = NULL;
//	inputMask[1] = NULL;
	realMask = NULL;
	homMat[0] = NULL;
	homMat[1] = NULL;
}

//Destructs every allocated memory by this class
OkanMos::~OkanMos()
{
	if ( homMat[0] != NULL )
		cvReleaseMat( &homMat[0] );
	if ( homMat[1] != NULL )
		cvReleaseMat( &homMat[1] );
}

void OkanMos::getMosaic( IplImage ** im , uchar *** mask , double imLine[6] )
{
	*im = image[0];
	*mask = realMask;
	if ( imLine != NULL )
		for ( int i = 0 ; i < 6 ; i++ )
			imLine[i] = line[i];
	return;
}

// Take clone of i0 and make it base image. Allocate space for the mask and initialize it. 
void OkanMos::setBaseImage( IplImage* i0 , uchar** i0Mask , double i0line[6] )
{
	image[0] = cvCloneImage( i0 );
	// Allocate memory for realMask
	realMask = new uchar*[ i0->height ];
	for ( int i = 0 ; i < i0->height ; i++ )
	{
		realMask[i] = new uchar[ i0->width ];
	}
	// Fill it with 1
	if ( i0Mask == NULL )
	{
		for ( int j = 0 ; j < i0->height ; j++ )
		{
			for ( int i = 0 ; i < i0->width ; i++ )
			{
				realMask[j][i] = 1;
			}
		}
	}
	else
	{
		for ( int j = 0 ; j < i0->height ; j++ )
		{
			for ( int i = 0 ; i < i0->width ; i++ )
			{
				realMask[j][i] = i0Mask[j][i];
			}
		}
	}

	// Intialize shadeMul and shadeAdd
	for ( int i = 0 ; i < 3 ; i++ )
	{
		shadeMul[i] = 1;
		shadeAdd[i] = 0;
	}
	image[1] = NULL;
	if ( homMat[0] != NULL )
		cvReleaseMat( &homMat[0] );
	if ( homMat[1] != NULL )
		cvReleaseMat( &homMat[1] );
	homMat[0] = NULL;
	homMat[1] = NULL;

	//world file information
	if ( i0line != NULL )
		for ( int i = 0 ; i < 6 ; i++ )
			line[i] = i0line[i];
	else
		for ( int i = 0 ; i < 6 ; i++ )
			line[i] = 0;
	return;
}

//Takes clone of all parameters and add it to Mosaic.
void OkanMos::addSucImage( IplImage* suc , CvMat* mbase , CvMat* msuc , int interpolationType ,  uchar** i1Mask )
{
	// Calculate shadeAdd and shadeMul for illumination change
	if ( image[1] == NULL )
	{
		removeShade( image[0] , suc , mbase , msuc );
	}
	else
	{
		removeShade( image[1] , suc , mbase , msuc );
	}
	image[1] = suc;
	if ( homMat[0] == NULL )
	{
		homMat[0] = cvCloneMat( mbase );
		homMat[1] = cvCloneMat( msuc );
	}
	else
	{
		CvMat* tmpMat0 = cvCloneMat( homMat[0] );
		CvMat* tmpMat1 = cvCloneMat( homMat[1] );
		cvMatMul( tmpMat0 , mbase , homMat[0] );
		cvMatMul( msuc , tmpMat1 , homMat[1] );
		cvReleaseMat( &tmpMat0 );
		cvReleaseMat( &tmpMat1 );
	}
	makeMosaic( interpolationType , i1Mask );
}

//Calculates new shadeAdd and shadeMul values for new add image
void OkanMos::removeShade( IplImage* &i0 , IplImage* &i1 , CvMat* &m0 , CvMat* &m1 )
{
	double mul[3] , add[3];
	RgbImage hom0( i0 );
	RgbImage hom1( i1 );
	float sa, x , y , *h;
	int step;
	CvSize size;
	CvScalar mean0 , sdv0 , mean1 , sdv1;
	// Allocate meory for mask0 and mask1 and initialize them
	uchar **mask0 = new uchar*[ i0->height ];
	for ( int i = 0 ; i < i0->height ; i++ )
	{
		mask0[i] = new uchar[ i0-> width ];
	}
	for ( int j = 0 ; j < i0->height ; j++ )
	{
		for ( int i = 0 ; i < i0->width ; i++ )
		{
			mask0[j][i] = 1;
		}
	}
	uchar **mask1 = new uchar*[ i1->height ];
	for ( int i = 0 ; i < i1->height ; i++ )
	{
		mask1[i] = new uchar[ i1-> width ];
	}
	for ( int j = 0 ; j < i1->height ; j++ )
	{
		for ( int i = 0 ; i < i1->width ; i++ )
		{
			mask1[j][i] = 1;
		}
	}
	
	// determine pixels which are overllaping with other image
	CvMat matIm0, matIm1;
	cvInitMatHeader( &matIm0 , i0->height , i0->width , CV_8UC1 , mask0 );
	cvInitMatHeader( &matIm1 , i1->height , i1->width , CV_8UC1 , mask1 );
	cvGetRawData( m0 , (uchar**)&h, &step, &size );
	for ( int j = 0 ; j < i0->height ; j++ )
	{
		for ( int i = 0 ; i < i0->width ; i++ )
		{
			sa = i * h[6] + j * h[7] + h[8];
			x = ( i * h[0] + j * h[1] + h[2] ) / sa;
			y = ( i * h[3] + j * h[4] + h[5] ) / sa;
			if ( ( x > 0 && x < i1->width ) && ( y > 0 && y < i1->height ) )
				mask0[j][i] = 1;
			else
				mask0[j][i] = 0;
		}
	}

	cvAvgSdv( i0 , &mean0 , &sdv0 , &matIm0 );
	cvGetRawData( m1 , (uchar**)&h, &step, &size );
	for ( int j = 0 ; j < i1->height ; j++ )
	{
		for ( int i = 0 ; i < i1->width ; i++ )
		{
			sa = i * h[6] + j * h[7] + h[8];
			x = ( i * h[0] + j * h[1] + h[2] ) / sa;
			y = ( i * h[3] + j * h[4] + h[5] ) / sa;
			if ( ( x > 0 && x < i0->width ) && ( y > 0 && y < i0->height ) )
				mask1[j][i] = 1;
			else
				mask1[j][i] = 0;
		}
	}
	cvAvgSdv( i1 , &mean1 , &sdv1 , &matIm1 );
	for( int i = 0 ; i < 3 ; i++ )
	{
		mul[i] = sdv0.val[i] / sdv1.val[i];
		add[i] = mean0.val[i] - mean1.val[i] * mul[i];
		shadeAdd[i] += shadeMul[i]*add[i];
		shadeMul[i] =shadeMul[i]*mul[i];
	}
	//Deallocate memory
	for( int i = 0 ; i < i0->height ; i++ )
		delete[] mask0[i];
	delete[] mask0;
	for( int i = 0 ; i < i1->height ; i++ )
		delete[] mask1[i];
	delete[] mask1;
}


//Core function which makes mosaic by adding image[1] to image[0]
void OkanMos::makeMosaic( int interpolationType , uchar** im1Mask )
{
	RgbImage im0( image[0] );
	RgbImage im1( image[1] );
	int min[2] = { 0 , 0 };
	int max[2] = { image[0]->width - 1 , image[0]->height - 1 };
	double sa , x , y;
	float *h;
	cvGetRawData( homMat[1] , (uchar**)&h );
	for( int i = 0 ; i < image[1]->width ; i += image[1]->width - 1 )
	{
		for ( int j = 0 ; j < image[1]->height ; j += image[1]->height - 1 )
		{
			sa = i * h[6] + j * h[7] + h[8];
			x = ( i * h[0] + j * h[1] + h[2] ) / sa;
			y = ( i * h[3] + j * h[4] + h[5] ) / sa;
			if ( x < min[0] )
				min[0] = cvCeil(x);
			else if ( x > max[0] )
				max[0] = cvFloor(x);
			if ( y < min[1] )
				min[1] = cvCeil(y);
			else if ( y > max[1] )
				max[1] = cvFloor(y);
		}
	}
	line[4] -= min[0] * line[0];
	line[5] -= min[1] * line[3];
	IplImage* newImage = cvCreateImage( cvSize( max[0] - min[0] + 1 , max[1] - min[1] + 1), IPL_DEPTH_8U , 3 );
	RgbImage newIm( newImage );
	uchar** newMask = new uchar*[ newImage->height ];
	for ( int i = 0 ; i < newImage->height ; i++ )
	{
		newMask[i] = new uchar[ newImage->width ];
	}
	for ( int j = 0 ; j < newImage->height ; j++ )
	{
		for ( int i = 0 ; i < newImage->width ; i++ )
		{
			newMask[j][i] = 0;
		}
	}
	cvGetRawData( homMat[0] , (uchar**)&h );
	double yLower, yUpper, xLower , xUpper;
	uchar pixels[2][2][3];
	double tmp[3];
	int yFloor, yCeil , xFloor , xCeil;
	for( int j = min[1] ; j <= max[1] ; j++ )
	{
		for( int i = min[0] ; i <= max[0] ; i++ )
		{
			if ( ( i >= 0 && i < image[0]->width ) && ( j >= 0 && j < image[0]->height ) && realMask[j][i] == 1 )
			{
				newIm[ j - min[1] ][ i - min[0] ] = im0[j][i];
				newMask[ j - min[1] ][ i - min[0] ] = 1;
			}
			else
			{
				sa = i * h[6] + j * h[7] + h[8];
				x = ( i * h[0] + j * h[1] + h[2] ) / sa;
				y = ( i * h[3] + j * h[4] + h[5] ) / sa;
				if ( ( x > 0 && x < image[1]->width - 1 ) && ( y > 0 && y < image[1]->height - 1 ) )
				{
					if ( interpolationType == NEAR_NEIG )
					{
						if ( im1Mask != NULL && im1Mask[ cvRound( y ) ][ cvRound( x ) ] == 0 )
							goto EMPTYPIXEL;
					newIm[ j - min[1] ][ i - min[0] ].b = makeUImagePixel( shadeMul[0] * im1[ cvRound( y ) ][ cvRound( x ) ].b + shadeAdd[0] );
					newIm[ j - min[1] ][ i - min[0] ].g = makeUImagePixel( shadeMul[1] * im1[ cvRound( y ) ][ cvRound( x ) ].g + shadeAdd[1] );
					newIm[ j - min[1] ][ i - min[0] ].r = makeUImagePixel( shadeMul[2] * im1[ cvRound( y ) ][ cvRound( x ) ].r + shadeAdd[2] );
					}
					else if ( interpolationType == BILINEAR )
					{
						yFloor = cvFloor( y );
						yCeil = cvCeil( y );
						xFloor = cvFloor( x );
						xCeil = cvCeil( x );

						yLower = y - yFloor;
						yUpper = yCeil - y;
						xLower = x - xFloor;
						xUpper = xCeil - x;

						if ( yLower == yUpper )
							++yUpper;
						if ( xLower == xUpper )
							++xUpper;
						if ( im1Mask != NULL && im1Mask[ yFloor ][ xFloor ] == 0 && im1Mask[ yFloor ][ xCeil ] == 0 && im1Mask[ yCeil ][ xFloor ] == 0 && im1Mask[ yCeil ][ xCeil ] == 0 )
							goto EMPTYPIXEL;

						pixels[0][0][0] = im1[ yFloor ][ xFloor ].b;
						pixels[0][0][1] = im1[ yFloor ][ xFloor ].g;
						pixels[0][0][2] = im1[ yFloor ][ xFloor ].r;
						pixels[0][1][0] = im1[ yFloor ][ xCeil ].b;
						pixels[0][1][1] = im1[ yFloor ][ xCeil ].g;
						pixels[0][1][2] = im1[ yFloor ][ xCeil ].r;
						pixels[1][0][0] = im1[ yCeil ][ xFloor ].b;
						pixels[1][0][1] = im1[ yCeil ][ xFloor ].g;
						pixels[1][0][2] = im1[ yCeil ][ xFloor ].r;
						pixels[1][1][0] = im1[ yCeil ][ xCeil ].b;
						pixels[1][1][1] = im1[ yCeil ][ xCeil ].g;
						pixels[1][1][2] = im1[ yCeil ][ xCeil ].r;

						for( int k = 0 ; k < 3 ; k++ )
						{
							tmp[k] = ( pixels[0][0][k] * xUpper + pixels[0][1][k] * xLower ) * yUpper + 
								( pixels[1][0][k] * xUpper + pixels[1][1][k] * xLower ) * yLower;
						}
						newIm[ j - min[1] ][ i - min[0] ].b = makeUImagePixel( shadeMul[0] * tmp[0] + shadeAdd[0] );
						newIm[ j - min[1] ][ i - min[0] ].g = makeUImagePixel( shadeMul[1] * tmp[1] + shadeAdd[1] );
						newIm[ j - min[1] ][ i - min[0] ].r = makeUImagePixel( shadeMul[2] * tmp[2] + shadeAdd[2] );
					}
					else
						;
					newMask[ j - min[1] ][ i - min[0] ] = 1;
				}
				else
				{
EMPTYPIXEL:
					newIm[ j - min[1] ][ i - min[0] ].b = 0;
					newIm[ j - min[1] ][ i - min[0] ].g = 0;
					newIm[ j - min[1] ][ i - min[0] ].r = 0;
					newMask[ j - min[1] ][ i - min[0] ] = 0;
				}
			}
		}
	}
	for ( int i = 0 ; i < image[0]->height ; i++ )
		delete[] realMask[i];
	delete[] realMask;
	cvReleaseImage( &image[0] );
	image[0] = newImage;
	realMask = newMask;
	if ( min[0] < 0 || min[1] < 0 )
	{
		CvMat* tmpMat = cvCreateMat( 3 , 3 , CV_32FC1 );
		cvSetIdentity( tmpMat );
		cvGetRawData( tmpMat , (uchar**)&h );
		h[2] = min[0];
		h[5] = min[1];
		CvMat* tmpMat2 = cvCloneMat( homMat[0] );
		cvMatMul( tmpMat2 , tmpMat , homMat[0] );
		cvInvert( homMat[0] , homMat[1] );
		cvReleaseMat( &tmpMat );
		cvReleaseMat( &tmpMat2 );
	}
}

inline uchar OkanMos::makeUImagePixel( double x)
{
	if ( x < 0 )
		return 0;
	else if ( x > 255 )
		return 255;
	else
		return cvRound( x );
}
