#include "OkanOrto.h"
#include "stdio.h"
#include <float.h>
#include <string>
#include <iostream>

using namespace std;

OkanOrto::OkanOrto()
{
	min[0] = DBL_MAX;
	max[0] = DBL_MIN;
	min[1] = DBL_MAX;
	max[1] = DBL_MIN;
}

OkanOrto::~OkanOrto()
{
}

void OkanOrto::setImage( IplImage* im , uchar **new_inputMask, const OkanImageParam & new_imageParam , double new_avgHeight)
{
	// setimage
	image.push_back( im );
	inputMask.push_back( new_inputMask );
	imageParam.push_back( new_imageParam );
	averageHeight.push_back( new_avgHeight );
	int index = image.size() - 1;

	//createorto
	double s, x , y;
	double r[3][3];

	double** tmportoX = new double*[ image[index]->height ];
	for ( int i = 0 , size = image[index]->height ; i < size ; i++ )
		tmportoX[i] = new double[ image[index]->width ];

	double** tmportoY = new double*[ image[index]->height ];
	for ( int i = 0 , size = image[index]->height ; i < size ; i++ )
		tmportoY[i] = new double[ image[index]->width ];

	ortoX.push_back( tmportoX );
	ortoY.push_back( tmportoY );

	double Omega = imageParam[index].omega;
	double Phi = imageParam[index].phi;
	double Kappa = imageParam[index].kappa;

	r[0][0] = cos(Phi)*cos(Kappa);
	r[0][1] = sin(Omega)*sin(Phi)*cos(Kappa) + cos(Omega)*sin(Kappa);
	r[0][2] = -cos(Omega)*sin(Phi)*cos(Kappa) + sin(Omega)*sin(Kappa);
	r[1][0] = -cos(Phi)*sin(Kappa);
	r[1][1] = -sin(Omega)*sin(Phi)*sin(Kappa) + cos(Omega)*cos(Kappa);
	r[1][2] = cos(Omega)*sin(Phi)*sin(Kappa) + sin(Omega)*cos(Kappa);
	r[2][0] = sin(Phi);
	r[2][1] = -sin(Omega)*cos(Phi);
	r[2][2] = cos(Omega)*cos(Phi);

	double averageHeight_camZ = averageHeight[index] - imageParam[index].camZ;
	y = ( image[index]->height / 2 - 0 - imageParam[index].principalPointY ) * imageParam[index].pixelSize;
	for ( int j = 0 ; j < image[index]->height ; j++ , y -= imageParam[index].pixelSize )
	{
		x = ( 0 - image[index]->width / 2 - imageParam[index].principalPointX ) * imageParam[index].pixelSize;
		for ( int i = 0 ; i < image[index]->width ; i++ , x += imageParam[index].pixelSize )
		{
			if ( inputMask[index] != NULL && inputMask[index][j][i] == 0 )
				continue;

			s = averageHeight_camZ / ( r[0][2] * x + r[1][2] * y + r[2][2] * imageParam[index].f );
			ortoX[index][j][i] = imageParam[index].camX + s * ( r[0][0] * x + r[1][0] * y + r[2][0] * imageParam[index].f );
			ortoY[index][j][i] = imageParam[index].camY + s * ( r[0][1] * x + r[1][1] * y + r[2][1] * imageParam[index].f );

			if ( ortoX[index][j][i] < min[0] )
				min[0] = ortoX[index][j][i];
			if ( ortoX[index][j][i] > max[0] )
				max[0] = ortoX[index][j][i];
			if ( ortoY[index][j][i] < min[1] )
				min[1] = ortoY[index][j][i];
			if ( ortoY[index][j][i] > max[1] )
				max[1] = ortoY[index][j][i];
		}
	}
}

void OkanOrto::giveOrtoImage( IplImage* & newImage , uchar** & outputMask , double line[6] )
{
	int height = 0, width = 0;
	for ( int i = 0 , size = image.size() ; i < size ; i++ )
	{
		height += image[i]->height;
		width += image[i]->width;
	}
	coreGiveOrtoImage( newImage , outputMask , line , width , height );
}

void OkanOrto::giveOrtoImageByPixelSize( IplImage* & newImage , uchar** & outputMask , double line[6] , double pixelSizeX , double pixelSizeY )
{
	int newWidth = (int) ( ( max[0] - min[0] ) / pixelSizeX );
	int newHeight = (int) ( ( max[1] - min[1] ) / pixelSizeY );
	if ( newWidth <= 0 || newHeight <= 0)
		throw string("Resolutions are not appropriate");
	coreGiveOrtoImage( newImage , outputMask , line , newWidth , newHeight );
}

void OkanOrto::giveOrtoImageByResolution( IplImage* & newImage , uchar** & outputMask , double line[6] , int resX , int resY )
{
	coreGiveOrtoImage( newImage , outputMask , line , resX , resY );
}

void OkanOrto::coreGiveOrtoImage( IplImage* & newImage , uchar** & outputMask , double line[6] , int newWidth , int newHeight )
{
	double pixDif[2];
	unsigned newX , newY;
	int tmp;

	pixDif[0] = ( max[0] - min[0] ) / newWidth ;	
	pixDif[1] = ( max[1] - min[1] ) / newHeight ;
	
	newImage = cvCreateImage( cvSize( newWidth , newHeight ) , IPL_DEPTH_8U , 3 );

	outputMask = new uchar*[newImage->height];
	for ( int i = 0 , size = newImage->height ; i < size ; i++ )
		outputMask[i] = new uchar[ newImage->width ];
	for ( int j = 0 ; j < newImage->height ; j++ )
	{
		for ( int i = 0 ; i < newImage->width ; i++ )
		{
			outputMask[j][i] = 0;
		}
	}

	RgbImage des( newImage );

	for ( int index = 0 , size = image.size() ; index < size ;index++ )
	{
		RgbImage src( image[index] );

		for ( int j = 0 ; j < image[index]->height ; j++ )
		{
			for ( int i = 0 ; i < image[index]->width ; i++ )
			{
				if ( inputMask[index] != NULL && inputMask[index][j][i] == 0 )
					continue;
				tmp = (int) ( ( ortoX[index][j][i] - min[0] ) / pixDif[0] );
				if ( tmp  < 0 )
					newX = 0;
				else if ( tmp >= newWidth )
					newX = newWidth - 1;
				else
					newX = tmp;

				tmp = (int) ( ( ortoY[index][j][i] - min[1] ) / pixDif[1] );
				if ( tmp  < 0 )
					newY = 0;
				else if ( tmp >= newHeight )
					newY = newHeight - 1;
				else
					newY = tmp;

				if ( outputMask[newY][newX] != 1 )
				{
					des[newY][newX].b = src[j][i].b;
					des[newY][newX].g = src[j][i].g;
					des[newY][newX].r = src[j][i].r;
					outputMask[newY][newX] = 1;
				}
			}
		}
	}
	
	line[0] = pixDif[0];
	line[1] = 0;
	line[2] = 0;
	line[3] = -pixDif[1];
	line[4] = min[0];
	line[5] = min[1];

/*	int begin , end;
	double earthX , earthY;
	int photoX, photoY;
	for ( int j = 0 ; j < newImage->height ; j++ )
	{
		earthY = j * pixDif[1] + min[1];
		begin = newImage->width - 1;
		for ( int i = 0 ; i < newImage->width ; i++ )
		{
			if ( outputMask )
			{
				begin = i;
				break;
			}
		}
		end = 0;
		for ( int i = newImage->width - 1 ; i >= 0 ; i-- )
		{
			if ( outputMask )
			{
				end = i;
				break;
			}
		}
		for ( int i = begin ; i <= end ; i++ )
		{
			if ( !outputMask )
			{
				earthX = i * pixDif[0] + min[0];
			}
		}
	}*/
}
