#include <wx/wx.h>
#include "convolutionWindow.h"

BEGIN_EVENT_TABLE(ConvolutionWindow, wxDialog)
EVT_COMBOBOX(IdPresets, ConvolutionWindow::onPresetSelection)
END_EVENT_TABLE()

ConvolutionWindow::ConvolutionWindow(wxWindow* parent, wxWindowID id)
  : wxDialog(parent, id, _("Convolution Window"), wxDefaultPosition, wxSize(250, 170)) {
  wxBoxSizer* outSizer = new wxBoxSizer(wxVERTICAL);
  SetSizer(outSizer);
  x11 = x12 = x13 = x21 = x22 = x23 = x31 = x32 = x33 = 0;
  wxComboBox* presetList = new wxComboBox(this, IdPresets);
  presetList->Append(_("Light Blur"));
  presetList->Append(_("Horizontal Sobel Operator"));
  presetList->Append(_("Vertical Sobel Operator"));
  presetList->Append(_("High Boost Filter"));

  outSizer->Add(presetList, 0, wxALL | wxEXPAND, 3);
  wxGridSizer* gridSizer = new wxGridSizer(3, 3, 3, 3);
  outSizer->Add(gridSizer, 0, wxALIGN_CENTER | wxALL, 5);
  for(unsigned int i = 0; i < 9; i++) {
    gridSizer->Add(new wxTextCtrl(this, IdX11 + i));
  }
  wxBoxSizer* buttonSizer = new wxBoxSizer(wxHORIZONTAL);
  outSizer->Add(buttonSizer, 0, wxALIGN_RIGHT | wxALL, 3);
  buttonSizer->Add(new wxButton(this, wxID_OK, _("OK")), 0);
  buttonSizer->Add(new wxButton(this, wxID_CANCEL, _("Cancel")), 0);
   
}

bool ConvolutionWindow::TransferDataFromWindow() {
  wxTextCtrl* ctrl = (wxTextCtrl*)FindWindow(IdX11);
  x11 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX12);
  x12 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX13);
  x13 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));
  
  ctrl = (wxTextCtrl*)FindWindow(IdX21);
  x21 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX22);
  x22 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX23);
  x23 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX31);
  x31 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX32);
  x32 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));

  ctrl = (wxTextCtrl*)FindWindow(IdX33);
  x33 = atof((const char*) wxConvCurrent->cWX2MB(ctrl->GetValue()));
  return true;
}

bool ConvolutionWindow::TransferDataToWindow() {
  char arr[128];

  sprintf(arr, "%0.5f", x11);
  ((wxTextCtrl*)FindWindow(IdX11))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x12);
  ((wxTextCtrl*)FindWindow(IdX12))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x13);
  ((wxTextCtrl*)FindWindow(IdX13))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x21);
  ((wxTextCtrl*)FindWindow(IdX21))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x22);
  ((wxTextCtrl*)FindWindow(IdX22))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x23);
  ((wxTextCtrl*)FindWindow(IdX23))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x31);
  ((wxTextCtrl*)FindWindow(IdX31))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x32);
  ((wxTextCtrl*)FindWindow(IdX32))->SetValue(wxString(arr, wxConvUTF8));

  sprintf(arr, "%0.5f", x33);
  ((wxTextCtrl*)FindWindow(IdX33))->SetValue(wxString(arr, wxConvUTF8));

  return true;
}

void ConvolutionWindow::onPresetSelection(wxCommandEvent& evt) {
  if(!((wxComboBox*)FindWindow(IdPresets))->GetValue().compare(_("Light Blur"))) {
    x11 = x12 = x13 = x21 = x22 = x23 = x31 = x32 = x33 = 0.111;
  }
  if(!((wxComboBox*)FindWindow(IdPresets))->GetValue().compare(_("Horizontal Sobel Operator"))) {
    x11 = x31 = -1; x21 = -2;
    x13 = x33 = 1; x23 = 2;
    x12 = x22 = x32 = 0;
  }

  if(!((wxComboBox*)FindWindow(IdPresets))->GetValue().compare(_("Vertical Sobel Operator"))) {
    x11 = x13 = -1; x12 = -2;
    x31 = x33 = 1; x32 = 2; 
    x21 = x22 = x23 = 0;
  }

  if(!((wxComboBox*)FindWindow(IdPresets))->GetValue().compare(_("High Boost Filter"))) {
    x12 = x32 = x21 = x23 = -1;
    x22 = 6;
    x11 = x13 = x31 = x33 = 0;
  }

  TransferDataToWindow();
}
