#include <wx/wx.h>

#include "mosaickerPanel.h"
#ifdef LINUX
#include "../projectManager.h"
//#include "../OkanReg.h"
#include "../OkanMos.h"
#include "../projectPresenter.h"
#include "../project.h"
#include "../content.h"
#include "../imagePane.h"
#include "../imageFactory.h"
#elif WIN32
#include "projectManager.h"
//#include "OkanReg.h"
#include "OkanMos.h"
#include "projectPresenter.h"
#include "project.h"
#include "content.h"
#include "imagePane.h"
#include "imageFactory.h"
#endif

#include <vector>
#include <string>
#include <iostream>
#include <map>
#include <cv.h>

using namespace std;

BEGIN_EVENT_TABLE(MosaickerPanel, wxWindow)
EVT_BUTTON(IdStartCtrl, MosaickerPanel::onStartMosaic)
END_EVENT_TABLE()

#ifdef LINUX
void plugMosaicker(wxWindow* parent, ProjectManager* pm)
#elif WIN32
  PLUGIN_API void __cdecl plugMosaicker(wxWindow* parent, ProjectManager* pm)
#endif
{
  wxBoxSizer* newSizer = new wxBoxSizer(wxVERTICAL);
  MosaickerPanel* mosaicker = new MosaickerPanel(pm, parent, wxID_ANY, wxDefaultPosition, wxSize(500, 550));
  parent->SetSizer(newSizer);
  newSizer->Add(mosaicker, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);
  return ;
}

#ifdef LINUX
wxSize negotiateSize()
#elif WIN32
PLUGIN_API wxSize __cdecl negotiateSize()
#endif
{
  return wxSize(xsize, ysize);
}

MosaickerPanel::MosaickerPanel(ProjectManager* pm, wxWindow* parent, wxWindowID id, wxPoint pos, wxSize size, long style) : 
  wxPanel(parent, id, pos, size, style) {
  this->pm = pm;

  nCorners = 128;
  cornerQLevel = 0;
  minCornerDistance = 0;
  blockSize = 10;
  aptSize = 5;
  cornerWinSize = 10;
  maxDistance = 1000;
  thresh = 0;
  nIter = 32;
  errTol = 1;

  CreateControls();
}
void MosaickerPanel::CreateControls() {
  wxBoxSizer* topSizer = new wxBoxSizer(wxVERTICAL);
  this->SetSizer(topSizer);

  wxBoxSizer* boxSizer = new wxBoxSizer(wxVERTICAL);
  topSizer->Add(boxSizer, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 5);

  //wxStaticText* desc = new wxStaticText(this, wxID_STATIC, _("Please choose the images to mosaic"), wxDefaultPosition, wxDefaultSize, 0);
  //boxSizer->Add(desc, 0, wxALIGN_LEFT | wxALL, 5);
  
#ifdef LINUX
  imListCtrl = new wxListCtrl(this, wxID_ANY, wxDefaultPosition, wxSize(350, 100), wxSIMPLE_BORDER);
#elif WIN32
  imListCtrl = new wxListCtrl(this, wxID_ANY, wxDefaultPosition, wxSize(350, 100));
#endif
  ProjectPresenter* pp = pm->getProjectPresenter();
  vector<pair<string,ContentType> >* contents = pp->enumerateContents();
  unsigned int c = 0;
  for(vector<pair<string, ContentType> >::iterator i = contents->begin(); i != contents->end(); i++, c++) {
    if((i->second == Image2D) || (i->second == Mosaic) || (i->second == OrthophotoType)) {
      imListCtrl->InsertItem(c, wxString(i->first.c_str(), wxConvUTF8));
    }
  }
  boxSizer->Add(imListCtrl, 0, wxALIGN_LEFT | wxALL, 3);

  wxComboBox* interpolationList = new wxComboBox(this, IdIPList); //, _(""), wxDefaultPosition, wxDefaultSize, wxCB_READONLY);
  interpolationList->Append(_("Nearest Neighbor Interpolation"));
  interpolationList->Append(_("Bilinear Interpolation"));
  interpolationList->Append(_("Bicubic Interpolation"));
  interpolationList->SetValue(_("Bicubic Interpolation"));
  boxSizer->Add(interpolationList, 0, wxALIGN_LEFT | wxALL, 3);

  //wxStaticText* harrisIntro = new wxStaticText(this, wxID_ANY, _("Harris corner detection parameters"));
  // boxSizer->Add(harrisIntro, wxALIGN_LEFT | wxALL, 3);

  wxGridSizer* gridSizer = new wxGridSizer(0, 2, 1, 1);
  //gridSizer->SetMinSize(300, 400);
  boxSizer->Add(gridSizer);//, 0, wxALIGN_CENTER_HORIZONTAL | wxALL , 3);
  wxStaticText* nCornersText = new wxStaticText(this, wxID_STATIC, _("# of Corners"));
  wxTextCtrl*  nCornersCtrl = new wxTextCtrl(this, IdNCorners, _(""));
  
  gridSizer->Add(nCornersText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer->Add(nCornersCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  
  wxStaticText* qLevelText = new wxStaticText(this, wxID_ANY, _("Quality Level Threshold"));
  wxTextCtrl*  qLevelCtrl = new wxTextCtrl(this, IdCornerQualityLevel);
  
  gridSizer->Add(qLevelText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer->Add(qLevelCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);

  wxStaticText* minDistText = new wxStaticText(this, wxID_ANY, _("Min Dist. Between Corners"));
  wxTextCtrl*  minDistCtrl = new wxTextCtrl(this, IdMinCornerDistance);
  
  gridSizer->Add(minDistText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer->Add(minDistCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  
  wxStaticText* blockSizeText = new wxStaticText(this, wxID_ANY, _("Block Size"));
  wxTextCtrl*  blockSizeCtrl = new wxTextCtrl(this, IdBlockSize);
  
  gridSizer->Add(blockSizeText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer->Add(blockSizeCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);

  wxStaticText* aptSizeText = new wxStaticText(this, wxID_ANY, _("Aperture Size"));
  wxTextCtrl*  aptSizeCtrl = new wxTextCtrl(this, IdApertureSize);
  
  gridSizer->Add(aptSizeText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer->Add(aptSizeCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);

  wxStaticText* putMatchesIntro = new wxStaticText(this, wxID_ANY, _("Putative Match Descriptors"));
  boxSizer->Add(putMatchesIntro, 0, wxALIGN_LEFT | wxALL, 1);

  wxGridSizer* gridSizer2 = new wxGridSizer(0, 2, 1, 1);
  boxSizer->Add(gridSizer2);//, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 3);

  wxStaticText* cornerWinSizeText = new wxStaticText(this, wxID_STATIC, _("Corner Window Size"));
  wxTextCtrl* cornerWinSizeCtrl = new wxTextCtrl(this, IdCornerWindowSize);

  gridSizer2->Add(cornerWinSizeText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer2->Add(cornerWinSizeCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  
  wxStaticText* maxDistanceText = new wxStaticText(this, wxID_STATIC, _("Max Dist. Between Matches"));
  wxTextCtrl* maxDistanceCtrl = new wxTextCtrl(this, IdMaxDistance);

  gridSizer2->Add(maxDistanceText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer2->Add(maxDistanceCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);

  wxStaticText* threshValText = new wxStaticText(this, wxID_STATIC, _("Threshold Value"));
  wxTextCtrl* threshValCtrl = new wxTextCtrl(this, IdThresholdValue);

  gridSizer2->Add(threshValText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer2->Add(threshValCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
 
  wxStaticText* ransacIntro = new wxStaticText(this, wxID_ANY, _("RANSAC Parameters"));
  boxSizer->Add(ransacIntro, 0, wxALIGN_LEFT | wxALL, 3);

  wxGridSizer* gridSizer3 = new wxGridSizer(0, 2, 1, 1);
  boxSizer->Add(gridSizer3);//, 0, wxALIGN_CENTER_HORIZONTAL | wxALL, 3);
  
  wxStaticText* nIterText = new wxStaticText(this, wxID_STATIC, _("# of Iterations"));
  wxTextCtrl* nIterCtrl = new wxTextCtrl(this, IdNIterations);

  gridSizer3->Add(nIterText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer3->Add(nIterCtrl, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  
  wxStaticText* errTolText = new wxStaticText(this, wxID_STATIC, _("Error Tolerance"));
  wxTextCtrl* errTolCtrl = new wxTextCtrl(this, IdErrorTolerance);

  gridSizer3->Add(errTolText, 0, wxALIGN_CENTER_VERTICAL | wxALL, 1);
  gridSizer3->Add(errTolCtrl, 0, wxALIGN_CENTER_VERTICAL | wxEXPAND | wxALL, 1);
  
  wxButton* startButton = new wxButton(this, IdStartCtrl, _("Mosaic"), wxDefaultPosition, wxDefaultSize, 0);
  boxSizer->Add(startButton, 0, wxALIGN_LEFT | wxALL, 5);

  gridSizer->RecalcSizes();
  gridSizer2->RecalcSizes();
  gridSizer3->RecalcSizes();
  boxSizer->RecalcSizes();
  topSizer->RecalcSizes();

  TransferDataToWindow();
}

void MosaickerPanel::onPaneClick(wxMouseEvent& evt) {
  vector<CvPoint2D32f> matches;
  
  if(evt.GetId() == IdSDialog) {
    matches = reg.getMatches(1);
  }
  else {
    matches = reg.getMatches(0);
  }

  int xUnit, yUnit, x, y;
  static int lastId = -1;
  ((ImagePanel*)FindWindow(evt.GetId()))->GetScrollPixelsPerUnit(&xUnit, &yUnit);
  ((ImagePanel*)FindWindow(evt.GetId()))->GetViewStart(&x, &y);
  x *= xUnit;
  y *= yUnit;

  x += evt.GetX();
  y += evt.GetY();

  bool matchFound = false;
  for(unsigned int i = 0; i < matches.size(); i++) {
    
    if((abs((long int)matches[i].x - x) < 5) 
       && (abs((long int)matches[i].y - y) < 5)) {
      try {
	cout << "Deleting a match: " <<  i << endl;
	reg.eraseMatch(i);
	((ImagePanel*)FindWindow(IdSDialog))->setMatches(reg.getMatches(1));
	((ImagePanel*)FindWindow(IdRDialog))->setMatches(reg.getMatches(0));
	((ImagePanel*)FindWindow(IdSDialog))->Refresh();
	((ImagePanel*)FindWindow(IdRDialog))->Refresh();
	matchFound = true;
      } 
      catch(string e) {
	cout << "Terminating the deletion with error: " << e << endl;
      }
      break;
    }
  }
  if(matchFound)
    return;

  CvPoint2D32f newMatch;
  if(lastId == -1) {
    newMatch.x = x;
    newMatch.y = y;
    lastId = evt.GetId();
    ((ImagePanel*)FindWindow(lastId))->addMatch(newMatch);
    ((ImagePanel*)FindWindow(lastId))->Refresh();
    return;
  }
  else {
    if(lastId == evt.GetId()) {
      ((ImagePanel*)FindWindow(lastId))->popMatch();
      ((ImagePanel*)FindWindow(lastId))->Refresh();
      lastId = -1;
      return;
    }
    else {
      newMatch.x = x;
      newMatch.y = y;
      lastId = -1;
      ((ImagePanel*)FindWindow(evt.GetId()))->addMatch(newMatch);
      ((ImagePanel*)FindWindow(evt.GetId()))->Refresh();
      if(evt.GetId() == IdSDialog) {      
	reg.addMatch( ((ImagePanel*)FindWindow(IdRDialog))->getMatch(), newMatch);
      }
      else {
	reg.addMatch(newMatch, ((ImagePanel*)FindWindow(IdSDialog))->getMatch());
      }
    }
  }
  
}
void MosaickerPanel::onStartMosaic(wxCommandEvent& evt) {
  bool selectROIs = false;
  bool roiSelected = false;
  vector<Content *> contentList;
  long item = -1;
  wxListItem info;
  ProjectPresenter* pp = pm->getProjectPresenter();
  map<string, Content*>* contents = pp->getContentsByName();
  while(1) {
    item = imListCtrl->GetNextItem(item, wxLIST_NEXT_ALL, wxLIST_STATE_SELECTED);
    if(item == -1)
      break;
    info.SetId(item);
    imListCtrl->GetItem(info);
    contentList.push_back((*contents->find((const char*)wxConvCurrent->cWX2MB(info.GetText()))).second);
  }
  
  if(contentList.size() < 2) {
    wxMessageDialog* noContent = new wxMessageDialog(this, _("You should select at least two images to perform mosaic."), _("Error"), wxICON_EXCLAMATION);
    noContent->ShowModal();
    return;
  }
  vector<IplImage*> iplArr;

  for(unsigned int i = 0; i < contentList.size(); i++) {
    static_cast<Image*>(contentList[i])->openImage();
    iplArr.push_back(static_cast<Image*>(contentList[i])->getIplHandle());
  }

  OkanMos* mos = new OkanMos;
  vector<ImagePanel*> imagePanes;
  ImagePanel* newPanel;

  //OkanReg reg;  
  reg.setImage( iplArr[contentList.size() - 1] , 0 );

  for(unsigned int i = contentList.size() - 1; i > 0; i--) {
    reg.setImage( iplArr[i - 1] , 1 );

    wxDialog* matchesDialog2 = new wxDialog(this, wxID_ANY, _("Select ROIs"), wxDefaultPosition, wxSize(620, 350));
    wxBoxSizer* outMatchesSizer = new wxBoxSizer(wxVERTICAL);
    matchesDialog2->SetSizer(outMatchesSizer);

    wxBoxSizer* topMatchesSizer = new wxBoxSizer(wxHORIZONTAL);
    outMatchesSizer->Add(topMatchesSizer);

    Image* referenceImage = ImageFactory::fromIplHandle(reg.getImage(0), "Reference Image");
    newPanel = new ImagePanel(referenceImage, matchesDialog2, IdRDialog, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxBORDER_DEFAULT, false, false);
    newPanel->SetInitialSize(wxSize(300,300));
    imagePanes.push_back(newPanel);

    Image* sensedImage = ImageFactory::fromIplHandle(reg.getImage(1), "Sensed Image");
    newPanel = new ImagePanel(sensedImage, matchesDialog2, IdSDialog, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL | wxBORDER_DEFAULT, false, false);
    newPanel->SetInitialSize(wxSize(300,300));
    imagePanes.push_back(newPanel);

    topMatchesSizer->Add(imagePanes[0], 1, wxALL | wxGROW, 3);
    topMatchesSizer->Add(imagePanes[1], 1, wxALL | wxGROW, 3);

    imagePanes[0]->Refresh();
    imagePanes[1]->Refresh();

    wxBoxSizer* matchesButtonSizer = new wxBoxSizer(wxHORIZONTAL);
    outMatchesSizer->Add(matchesButtonSizer);
    wxButton* doneButton = new wxButton(matchesDialog2, wxID_OK, _("OK"));
    wxButton* cancelButton = new wxButton(matchesDialog2, wxID_CANCEL, _("Cancel"));
    matchesButtonSizer->Add(doneButton, 0, wxALL | wxALIGN_RIGHT, 3);
    matchesButtonSizer->Add(cancelButton, 0, wxALL | wxALIGN_RIGHT, 3);

    if(selectROIs) {
      imagePanes[0]->setRoi(true);
      imagePanes[1]->setRoi(true);

      while(!roiSelected) {
	if(matchesDialog2->ShowModal() == wxID_OK) {
	  if(imagePanes[0]->roiExists && imagePanes[1]->roiExists) {
	    roiSelected = true;
	  }
	  else {
	    wxMessageDialog err(this, _("Please select a region of interest in both images."), _("Error"), wxICON_EXCLAMATION );
	    err.ShowModal();	    
	  }
	}
	else {
	  break;
	}
      }
      imagePanes[0]->roiExists = false;
      imagePanes[1]->roiExists = false;
      Roi rRoi = imagePanes[0]->roi;
      Roi sRoi = imagePanes[1]->roi;
      reg.setImageROI(0, rRoi.x, rRoi.y, rRoi.width, rRoi.height);
      reg.setImageROI(1, sRoi.x, sRoi.y, sRoi.width, sRoi.height);

      imagePanes[0]->setRoi(false);
      imagePanes[1]->setRoi(false);
      
    }
    
    //    double tmp = reg.findHarris( 0 , 22 , 0 , 10 );
    double tmp = reg.findHarris( 0 , nCorners , minCornerDistance , blockSize );
    //reg.findHarris( 1 , nCorners , minCornerDistance , blockSize );
    reg.findHarris( 1 , nCorners , tmp , blockSize );
    //reg.findHarris( 1 , 0 , tmp , 10 );

    reg.findPutMatch( aptSize , maxDistance , thresh );
    //reg.findPutMatch( 5 , 1000 , 0);
    

    matchesDialog2->SetTitle(_("Putative matches of reference and base images"));
    imagePanes[0]->Connect(IdRDialog, wxEVT_LEFT_DCLICK, wxMouseEventHandler(MosaickerPanel::onPaneClick), NULL, this);
    imagePanes[1]->Connect(IdSDialog, wxEVT_LEFT_DCLICK, wxMouseEventHandler(MosaickerPanel::onPaneClick), NULL, this);
    imagePanes[0]->setMatches(reg.getMatches(0));
    imagePanes[1]->setMatches(reg.getMatches(1));

    if(matchesDialog2->ShowModal() == wxID_CANCEL) {
      return;
    }

    reg.findRANSACHomography( nIter, errTol );
    //reg.findRANSACHomography( 32, 1 );

    if ( i == contentList.size() - 1 ) {
      unsigned char ** baseMask = ((Image*)contentList.back())->mask;
      double* baseWorldData = ((Image*)contentList.back())->getWorldData();
      mos->setBaseImage( reg.getImage( 0 ), baseMask );
    }
    unsigned char ** successorMask = ((Image*)contentList[i - 1])->mask;
    //double * successorWorldData = ((Image*)contentList[i - 1])->getWorldData();
    mos->addSucImage( reg.getImage(1) , reg.bestMat[0] , reg.bestMat[1] , BILINEAR, successorMask );
    //mos.addSucImage( reg.getImage(1) , reg.bestMat[0] ,
    //reg.bestMat[1] , NEAR_NEIG);

  }

  unsigned char *imData;
  int step = 0;
  CvSize roiSize;
  int width = mos->image[0]->width;
  int height = mos->image[0]->height;

  IplImage* outputMos;
  unsigned char** mask;
  double* outputWorldData = new double[6];
  mos->getMosaic(&outputMos, &mask, outputWorldData);
  cvGetRawData(outputMos, &imData, &step, NULL);
  unsigned char* rawData = new unsigned char[width*height*3];
  unsigned char* rp = imData;
  for(unsigned int i = 0; i < height; i++, rp += step) {
    for(unsigned int k = 0; k < width; k++) {
      rawData[(i*width*3) + 3 * k] = rp[3 * k + 2];
      rawData[(i*width*3) + 3 * k + 1] = rp[3 * k + 1];
      rawData[(i*width*3) + 3 * k + 2] = rp[3 * k];
    }
  }
  ImageHandle* newIm = new ImageHandle();
  newIm->width  = mos->image[0]->width;
  newIm->height = mos->image[0]->height;
  newIm->bpp = 3; 


  newIm->rawData = rawData;
  string comment;
  comment += "Mosaic of ";
  for(unsigned int i = 0; i < contentList.size(); i++) {
    comment += contentList[i]->name + ", ";
  }
  Mosaick* newImage = new Mosaick("New Mosaic", "INMEM", comment, "No world file");
  newImage->imHandle = newIm;
  newImage->mask = mask;
  if(((Image*)contentList.back())->getWorldData()) {
    ((Image*)contentList.back())->setWorldData(outputWorldData);
  }
  pm->addMosaicToProject(newImage);

  ((wxDialog*)(GetParent()->GetParent()))->EndModal(wxID_OK);

}

bool MosaickerPanel::TransferDataFromWindow() {
  wxTextCtrl* ctrl = (wxTextCtrl*)FindWindow(IdNCorners);
  
  ctrl->GetValue().ToLong(&nCorners);
  
  ctrl = (wxTextCtrl*)FindWindow(IdCornerQualityLevel);
  ctrl->GetValue().ToDouble(&cornerQLevel);

  ctrl = (wxTextCtrl*)FindWindow(IdMinCornerDistance);
  ctrl->GetValue().ToDouble(&minCornerDistance);
  
  ctrl = (wxTextCtrl*)FindWindow(IdBlockSize);
  ctrl->GetValue().ToDouble(&blockSize);

  ctrl = (wxTextCtrl*)FindWindow(IdApertureSize);
  ctrl->GetValue().ToDouble(&aptSize);

  ctrl = (wxTextCtrl*)FindWindow(IdCornerWindowSize);
  ctrl->GetValue().ToDouble(&cornerWinSize);

  ctrl = (wxTextCtrl*)FindWindow(IdMaxDistance);
  ctrl->GetValue().ToDouble(&maxDistance);

  ctrl = (wxTextCtrl*)FindWindow(IdThresholdValue);
  ctrl->GetValue().ToDouble(&thresh);

  ctrl = (wxTextCtrl*)FindWindow(IdNIterations);
  ctrl->GetValue().ToLong(&nIter);

  ctrl = (wxTextCtrl*)FindWindow(IdErrorTolerance);
  ctrl->GetValue().ToDouble(&errTol);
  
  return true;
}

bool MosaickerPanel::TransferDataToWindow() {
  wxTextCtrl* ctrl = (wxTextCtrl*)FindWindow(IdNCorners);
  ctrl->SetValue(wxString::Format(_("%d"), nCorners));

  ctrl = (wxTextCtrl*)FindWindow(IdCornerQualityLevel);
  ctrl->SetValue(wxString::Format(_("%1.5f"), cornerQLevel));

  ctrl = (wxTextCtrl*)FindWindow(IdMinCornerDistance);
  ctrl->SetValue(wxString::Format(_("%1.5f"), minCornerDistance));

  ctrl = (wxTextCtrl*)FindWindow(IdBlockSize);
  ctrl->SetValue(wxString::Format(_("%1.5f"), blockSize));

  ctrl = (wxTextCtrl*)FindWindow(IdApertureSize);
  ctrl->SetValue(wxString::Format(_("%1.5f"), aptSize));

  ctrl = (wxTextCtrl*)FindWindow(IdCornerWindowSize);
  ctrl->SetValue(wxString::Format(_("%1.5f"), cornerWinSize));

  ctrl = (wxTextCtrl*)FindWindow(IdMaxDistance);
  ctrl->SetValue(wxString::Format(_("%1.5f"), maxDistance));

  ctrl = (wxTextCtrl*)FindWindow(IdThresholdValue);
  ctrl->SetValue(wxString::Format(_("%1.5f"), thresh));

  ctrl = (wxTextCtrl*)FindWindow(IdNIterations);
  ctrl->SetValue(wxString::Format(_("%d"), nIter));

  ctrl = (wxTextCtrl*)FindWindow(IdErrorTolerance);
  ctrl->SetValue(wxString::Format(_("%1.5f"), errTol));

  return true;
}
