#include "globals.h"

#include "wx/panel.h"
#include "wx/wx.h"
#include "imagePane.h"

BEGIN_EVENT_TABLE(ImagePanel, wxPanel)
//EVT_PAINT(ImagePanel::onPaint)
EVT_LEFT_DOWN(ImagePanel::onMousePress)
EVT_LEFT_UP(ImagePanel::onMouseUp)
EVT_MOTION(ImagePanel::onMouseMove)
EVT_LEAVE_WINDOW(ImagePanel::onMouseMove)
EVT_SIZE(ImagePanel::onSize)
END_EVENT_TABLE()

void ImagePanel::onPaint(wxPaintEvent& evt) {
	if(!im)
		return;
	wxPaintDC dc(lowerPanel);
	//DoPrepareDC(dc);

	mainPanel->PrepareDC(dc);
	dc.BeginDrawing();

	dc.DrawBitmap(*im, 0, 0);

	dc.EndDrawing();	
}

void ImagePanel::onSize(wxSizeEvent& evt) {
	if(mainPanel)
		mainPanel->SetSize(evt.GetSize());

	if(fitToWindow) {
	  if(im->GetWidth() <= GetSize().GetWidth() &&  im->GetHeight() <= GetSize().GetHeight()) {
	    xScale = yScale = 1;
	    return ;
	  }
	  if(im->GetWidth() > GetSize().GetWidth())
	    xScale = (double) GetSize().GetWidth() / (double) im->GetWidth();
	  else
	    xScale = 1;
	  
	  if(im->GetHeight() > GetSize().GetHeight())	 
	    yScale = (double) GetSize().GetHeight() / (double) im->GetHeight();
	  else
	    yScale = 1;
	  Refresh();
	}	  
}

void ImagePanel::OnDraw(wxDC& dc) {
	if(!im)
		return;
	dc.BeginDrawing();

	dc.SetUserScale(xScale, yScale);
	dc.DrawBitmap(*im, 0, 0);

	dc.SetUserScale(1, 1);
	if(roiExists) {
	  dc.SetBrush(wxBrush(wxColour(45, 45, 200, 125), wxTRANSPARENT));
	  dc.SetPen(*wxGREY_PEN);
		      
	  dc.DrawRectangle(roi.x, roi.y, roi.width, roi.height);
	}

	if(!matches.empty()) {
	  for(unsigned int i = matches.size() - 1; i != -1; i--) {
	    dc.SetBrush(*wxRED_BRUSH);
	    dc.DrawCircle(matches[i].x, matches[i].y, 8);
	    dc.DrawText(wxString::Format(_("%d"), i), matches[i].x - 4, matches[i].y - 7);
	  }
	}

	dc.EndDrawing();	
}

void ImagePanel::onMousePress(wxMouseEvent& evt) {
  if(!canSelectRoi)
    return;
  if(roiExists && !mpressed) {
    roiExists = false;
    Refresh();
    //return ;
  }
  if(mpressed)
    return;
  
  int xUnit, yUnit;
  int xStart, yStart;
  GetScrollPixelsPerUnit(&xUnit, &yUnit);
  GetViewStart(&xStart, &yStart);
  
  roi.x = evt.GetX() + xStart * xUnit;
  roi.y = evt.GetY() + yStart * yUnit;
  mpressed = true;
}

void ImagePanel::onMouseMove(wxMouseEvent& evt) {
  if(!canSelectRoi)
    return;
  if(!mpressed)
    return ;
  roiExists = true;
  if(evt.Leaving()) {
    mpressed = false;
    Refresh();
    return;
  }
  int xUnit, yUnit;
  int xStart, yStart;
  int x,y;
  GetScrollPixelsPerUnit(&xUnit, &yUnit);
  GetViewStart(&xStart, &yStart);
  
  x = evt.GetX() + xStart * xUnit;
  y = evt.GetY() + yStart * yUnit;

  roi.width = x - roi.x;
  roi.height = y - roi.y;
  Refresh();
}

void ImagePanel::onMouseUp(wxMouseEvent& evt) {
  if(!canSelectRoi)
    return;
  mpressed = false;
}
