#ifndef _IMAGEPANE_H_
#define _IMAGEPANE_H_

#include "globals.h"
#include "wx/panel.h"
#include "content.h"
#include "OkanMisc.h"


class ImagePanel : public wxScrolledWindow {
private:
	enum { IdImageName = wxID_HIGHEST + 100,
		IdImageAddress,
		IdImageComments,
		IdUpperPanel,
		IdLowerPanel,
		IdMainPanel
	};
	wxBitmap* im;

	wxBoxSizer* mainSizer;
	wxBoxSizer* upperSizer;
	wxPanel* lowerPanel;
	wxPanel* upperPanel;
	wxPanel* mainPanel;
	  Image* imContent;
	  bool mpressed;
	  bool canSelectRoi;
	  bool fitToWindow;
 public:
	  Roi roi;
	  bool roiExists;
	  double xScale, yScale;
	  vector<CvPoint2D32f> matches;
public:
  ImagePanel(Image* imContent, wxWindow* parent, wxWindowID id, const wxPoint pos = wxDefaultPosition, const wxSize size = wxDefaultSize, long style = wxTAB_TRAVERSAL | wxBORDER_DEFAULT, bool canSelectRoi = false, bool fitToWindow = true) : wxScrolledWindow(parent, id, pos, size, style) {
		mainSizer = upperSizer = NULL;
		  mpressed = false;
		  roiExists = false;
		  this->canSelectRoi = canSelectRoi;
		  this->fitToWindow = fitToWindow;
		lowerPanel = upperPanel = mainPanel = NULL;
		im = NULL;
		  
		mainSizer = new wxBoxSizer(wxVERTICAL);
		  mainPanel = new wxPanel(this, IdMainPanel, wxPoint(2, 2), wxSize(size.GetWidth() - 4, size.GetHeight() - 4));
		upperPanel = new wxPanel(mainPanel, IdUpperPanel, wxPoint(5, 5), wxSize(mainPanel->GetSize().GetWidth() - 12, 100), wxBORDER_SIMPLE);
		lowerPanel = new wxPanel(mainPanel, IdLowerPanel, wxPoint(5, 110), wxSize(mainPanel->GetSize().GetWidth() - 12, mainPanel->GetSize().GetHeight() - 112), wxBORDER_SIMPLE);
		mainPanel->Show(false);
		upperPanel->SetMaxSize(wxSize(10000, 100));

		string imName("Image name: "), imAddress("Image url:"), imComment("Comments:");
		if(imContent) {
			imName += imContent->name;
			imAddress += imContent->address;
			imComment += imContent->comment;
			}
		wxStaticText* nameText = new wxStaticText(upperPanel, IdImageName, wxString(imName.c_str(), wxConvUTF8));
		wxStaticText* addressText = new wxStaticText(upperPanel, IdImageAddress, wxString(imAddress.c_str(), wxConvUTF8));
		wxStaticText* commentText = new wxStaticText(upperPanel, IdImageComments, wxString(imComment.c_str(),wxConvUTF8));

		upperSizer = new wxBoxSizer(wxVERTICAL);
		upperSizer->Add(nameText,0, wxALIGN_LEFT | wxALL, 5);
		upperSizer->Add(addressText, 0, wxALIGN_LEFT | wxALL, 5);
		upperSizer->Add(commentText,0, wxALIGN_LEFT | wxALL, 5);

		upperPanel->SetSizer(upperSizer);

		mainSizer->Add(upperPanel, 0, wxEXPAND | wxALL, 3);
		
		mainSizer->Add(lowerPanel, 1, wxEXPAND | wxALL, 3);
		mainPanel->SetSizer(mainSizer);
		mainPanel->SetAutoLayout(true);
		upperPanel->SetAutoLayout(true);
		mainPanel->Update();

		this->imContent = imContent;
		this->imContent->openImage();

		int nhSteps = ceil((float)imContent->getWidth() / 20.0);
		int nvSteps = ceil((float)imContent->getHeight() /  20.0);

		loadBitmap();
		  if(!fitToWindow) {
		    this->SetScrollbars(20,20,nhSteps,nvSteps);
		      xScale = 1;
		      yScale = 1;
		      }
		  else {
		    if(im->GetWidth() > GetSize().GetWidth())
		      xScale = (double) GetSize().GetWidth() / (double) im->GetWidth();
		    else
		      xScale = 1;
		    
		    if(im->GetHeight() > GetSize().GetHeight())	 
		      yScale = (double) GetSize().GetHeight() / (double) im->GetHeight();
		    else
		      yScale = 1;
		  }
		    
	}

	void setBitmap(wxBitmap* newIm) {
		im = newIm;
	
	}

	void setRoi(bool val) {
	  canSelectRoi = val;
	}

	DSize getZoom() {
	  DSize newSize;
	  newSize.x = xScale;
	  newSize.y = yScale;
	  return newSize;
	}

	void setZoom(double x, double y) {
	  fitToWindow = false;
	  xScale = x;
	  yScale = y;

	  int nhSteps = ceil((float)imContent->getWidth() * xScale / 20.0);
	  int nvSteps = ceil((float)imContent->getHeight() * yScale /  20.0);	     
	  this->SetScrollbars(20,20,nhSteps,nvSteps);		  
	}

	void setZoom(DSize newScale) {
	  fitToWindow = false;
	  xScale = newScale.x;
	  yScale = newScale.y;

	  int nhSteps = ceil((float)imContent->getWidth() * xScale / 20.0);
	  int nvSteps = ceil((float)imContent->getHeight() * yScale /  20.0);	     
	  this->SetScrollbars(20,20,nhSteps,nvSteps);		  

	}

	void setFitToWindow(bool val) {
	  fitToWindow = val;
	}

	Image* getImage() {
	  return imContent;
	}
	void loadBitmap() {

	    unsigned char* rawData;
	    imContent->getRawData(&rawData, NULL, NULL); 
	      wxImage* tmpIm = new wxImage(imContent->getWidth(), imContent->getHeight(), rawData, true);
		//tmpIm->SetData(rawData);

	  if(im)
	    delete im;
	  im = new wxBitmap(*tmpIm); 
	}

	  void loadBitmap(Image* newImage) {
	    imContent = newImage;
	    loadBitmap();
	  }

	  void setMatches(vector<CvPoint2D32f>& matches) {
	    this->matches = matches;
	  }

	  void addMatch(CvPoint2D32f newPoint) {
	    matches.push_back(newPoint);
	  }

	  void popMatch() {
	    matches.pop_back();
	  }
	  
	  CvPoint2D32f getMatch() {
	    return matches.back();
	  }
	void onPaint(wxPaintEvent&);
	void onSize(wxSizeEvent&);
	  void onMousePress(wxMouseEvent&);
	  void onMouseMove(wxMouseEvent&);
	  void onMouseUp(wxMouseEvent&);
	  
	virtual void OnDraw(wxDC&);

protected:
	DECLARE_EVENT_TABLE()
};
#endif
