#include <wx/wx.h>
#include "projectManagerWindow.h"
#include "imageProcessorWindow.h"
#include "convolutionWindow.h"
#include "content.h"
#include "imagePane.h"

BEGIN_EVENT_TABLE(ImageProcessorWindow, wxPanel)
EVT_COMMAND_SCROLL(IdBrightness, ImageProcessorWindow::onBrightness)
EVT_BUTTON(IdHistEq, ImageProcessorWindow::onHistEq)
EVT_BUTTON(IdConvolution, ImageProcessorWindow::onFilter)
EVT_BUTTON(IdZoomIn, ImageProcessorWindow::onZoomIn)
EVT_BUTTON(IdZoomOut, ImageProcessorWindow::onZoomOut)
EVT_BUTTON(IdRotateLeft, ImageProcessorWindow::onRotateLeft)
EVT_BUTTON(IdRotateRight, ImageProcessorWindow::onRotateRight)
END_EVENT_TABLE()

ImageProcessorWindow::ImageProcessorWindow(ProjectManagerWindow* pmw, wxWindow* parent, wxWindowID id)
  : wxPanel(parent, id) {
  this->pmw = pmw;
  this->pm = pmw->pm;

  wxBoxSizer* outSizer = new wxBoxSizer(wxHORIZONTAL);
  SetSizer(outSizer);
  wxGridSizer* leftmostSizer = new wxGridSizer(2, 2, 3, 3);
  outSizer->Add(leftmostSizer, 0, wxEXPAND | wxALL | wxALIGN_CENTER, 3);
  wxStaticText* brightnessText = new wxStaticText(this, wxID_ANY, _("Brightness"));
  leftmostSizer->Add(brightnessText, 0, wxEXPAND | wxALL | wxALIGN_CENTER_VERTICAL, 3);
  wxSlider* brightnessSlider = new wxSlider(this, IdBrightness, 0, -50, 50);
  leftmostSizer->Add(brightnessSlider, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL | wxALL, 3);

  wxStaticText* contrastText = new wxStaticText(this, wxID_ANY, _("Contrast"));
  leftmostSizer->Add(contrastText, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL | wxALL, 3);
  wxSlider* contrastSlider = new wxSlider(this, IdContrast, 0, -50, 50);
  leftmostSizer->Add(contrastSlider, 0, wxEXPAND | wxALIGN_CENTER_VERTICAL | wxALL, 3);
 
  wxBoxSizer* middleSizer = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(middleSizer, 0, wxALL | wxEXPAND | wxALIGN_CENTER, 3);
  /*wxButton* histEqButton = new wxButton(this, IdHistEq, _("Eq"));
  middleSizer->Add(histEqButton, 0, wxEXPAND | wxALL, 3);

  wxButton* convolveButton = new wxButton(this, IdConvolution, _("Filter"));
  middleSizer->Add(convolveButton, 0, wxEXPAND | wxALL, 3);
  */
  wxBitmap::InitStandardHandlers();

  wxBitmap* zoomInIcon = new wxBitmap(_("zoomIn.bmp"), wxBITMAP_TYPE_BMP);
  wxBitmapButton* zoomInButton = new wxBitmapButton(this, IdZoomIn, *zoomInIcon);
  zoomInButton->SetInitialSize(wxSize(24, 24));
  middleSizer->Add(zoomInButton, 0, wxEXPAND | wxALL, 3);

  wxBitmap* zoomOutIcon = new wxBitmap(_("zoomOut.bmp"), wxBITMAP_TYPE_BMP);
  wxBitmapButton* zoomOutButton = new wxBitmapButton(this, IdZoomOut, *zoomOutIcon);
  zoomOutButton->SetInitialSize(wxSize(24, 24));
  middleSizer->Add(zoomOutButton, 0, wxEXPAND | wxALL, 3);

  wxBoxSizer* middleSizer3 = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(middleSizer3, 0, wxALIGN_CENTER | wxEXPAND | wxALL, 5);

  wxBitmap* rotateRightIcon = new wxBitmap(_("rotateRight.bmp"), wxBITMAP_TYPE_BMP);
  wxBitmapButton* rotateRightButton = new wxBitmapButton(this, IdRotateRight, *rotateRightIcon);
  rotateRightButton->SetInitialSize(wxSize(24, 24));
  middleSizer3->Add(rotateRightButton, 0, wxEXPAND | wxALL, 3);

  wxBitmap* rotateLeftIcon = new wxBitmap(_("rotateLeft.bmp"), wxBITMAP_TYPE_BMP);
  wxBitmapButton* rotateLeftButton = new wxBitmapButton(this, IdRotateLeft, *rotateLeftIcon);
  rotateLeftButton->SetInitialSize(wxSize(24, 24));
  middleSizer3->Add(rotateLeftButton, 0, wxEXPAND | wxALL, 3);


  wxBoxSizer* middleSizer2 = new wxBoxSizer(wxVERTICAL);
  outSizer->Add(middleSizer2, 0, wxALIGN_CENTER | wxEXPAND | wxALL, 5);

  wxBitmap* filterIcon = new wxBitmap(_("filter.bmp"), wxBITMAP_TYPE_BMP);
  wxBitmapButton* filterButton = new wxBitmapButton(this, IdConvolution, *filterIcon);
  filterButton->SetInitialSize(wxSize(24, 24));
  middleSizer2->Add(filterButton, 0, wxEXPAND | wxALL, 3);

}

void ImageProcessorWindow::onBrightness(wxScrollEvent& evt){
  Content* content = pmw->getSelectedContent();
  Image* image = static_cast<Image*>(content);
  Image* newImage;
  ImageProcessor::adjustBrightness(image, &newImage, evt.GetPosition());

  pm->replaceContent(content->name, newImage);
  delete image;
  ImagePanel* pane = pmw->getSelectedPane();
  pane->loadBitmap(newImage);
  pane->Refresh();
}

void ImageProcessorWindow::onHistEq(wxCommandEvent& evt) {
  Content* content = pmw->getSelectedContent();
  Image* image = static_cast<Image*>(content);
  Image* newImage;
  ImageProcessor::equalizeHistogram(image, &newImage);
  
  pm->replaceContent(content->name, newImage);
  ImagePanel* pane = pmw->getSelectedPane();
  pane->loadBitmap(newImage);
  pane->Refresh();
}

void ImageProcessorWindow::onFilter(wxCommandEvent& evt) {
  ConvolutionWindow* cw = new ConvolutionWindow(this, wxID_ANY);
  if(cw->ShowModal() == wxID_OK) {
    Content* content = pmw->getSelectedContent();
    Image* image = static_cast<Image*>(content);
    Image* newImage;
    Point p;
    p.x = 2;
    p.y = 2;
    float mat[9];

    mat[0] = cw->x11;
    mat[1] = cw->x12;
    mat[2] = cw->x13;
    mat[3] = cw->x21;
    mat[4] = cw->x22;
    mat[5] = cw->x23;
    mat[6] = cw->x31;
    mat[7] = cw->x32;
    mat[8] = cw->x33;
    ImageProcessor::convolveImage(image, &newImage, mat, 3, 3, p, 0);
    pm->replaceContent(content->name, newImage);
    ImagePanel* pane = pmw->getSelectedPane();
    pane->loadBitmap(newImage);
    pane->Refresh();
  }
}

void ImageProcessorWindow::onZoomIn(wxCommandEvent& evt) {
  ImagePanel* imagePane = pmw->getSelectedPane();
  DSize oldZoom = imagePane->getZoom();
  DSize newZoom(oldZoom.x * 2, oldZoom.y * 2);
  imagePane->setZoom(newZoom); 
  imagePane->Refresh();
}

void ImageProcessorWindow::onZoomOut(wxCommandEvent& evt) {
  ImagePanel* imagePane = pmw->getSelectedPane();
  DSize oldZoom = imagePane->getZoom();
  DSize newZoom(oldZoom.x / 2.0, oldZoom.y  / 2.0);
  imagePane->setZoom(newZoom); 
  imagePane->Refresh();

}

void ImageProcessorWindow::onRotateRight(wxCommandEvent& evt) {
  Content* content = pmw->getSelectedContent();
  Image* image = static_cast<Image*>(content);
  Image* newImage;
  ImageProcessor::rotateRight(image, &newImage);
  pm->replaceContent(content->name, newImage);
  ImagePanel* pane = pmw->getSelectedPane();
  pane->loadBitmap(newImage);
  pane->Refresh();
}

void ImageProcessorWindow::onRotateLeft(wxCommandEvent& evt) {
  Content* content = pmw->getSelectedContent();
  Image* image = static_cast<Image*>(content);
  Image* newImage;
  ImageProcessor::rotateLeft(image, &newImage);
  pm->replaceContent(content->name, newImage);
  ImagePanel* pane = pmw->getSelectedPane();
  pane->loadBitmap(newImage);
  pane->Refresh();

}

void ImageProcessorWindow::setSliders(int newVal) {
  ((wxSlider*)FindWindow(IdBrightness))->SetValue(newVal);
  ((wxSlider*)FindWindow(IdContrast))->SetValue(newVal);
}
