#ifndef __PROJECTMANAGER_H__
#define __PROJECTMANAGER_H__
#include <utility>
#include <string>
#include <vector>

using namespace std;

#include "project.h"
#include "projectPresenter.h"
#include "historyChaser.h"
#include "content.h"

class ProjectManager {
 private:
  Project* project;
  HistoryChaser* hc;
 public:
  ProjectManager() {
    project = NULL;
    hc = NULL;
  }
 public:
  ///<summary>Closes a previously opened project and loads a new
  ///project into the workspace</summary>
  ///<remarks>Project Manager manages at most one project at a time.
  ///<param name="projectPath">The absolute path to the project file. </param>
  int openProject(string projectPath);
  int newProject(string projectName, vector<string>& pathList, bool absAdd, string projectPath = ".", bool copyFiles = true);
  int saveProject();
  int closeProject(bool saveProject = false);
  int saveAsProject(string projectPath);
  bool projectLoaded();
  bool projectModified();
  int renameProject(string newName);
  int addImageToProject(string imageName, string imagePath, string comment);
  int addImageToProject(Image* im);
  int addCameraToProject(Camera* camera);
  int addOrthophotoToProject(Orthophoto* op);
  int addMosaicToProject(Mosaick*);
  int addVideoToProject(Video*);
  int undo();
  int replaceContent(string, Image*);
  int removeContent(string);
  ProjectPresenter* getProjectPresenter();
};

#endif
