#include <wx/treectrl.h>
#include <wx/artprov.h>
#include <wx/imaglist.h>
#include <wx/mediactrl.h>
#ifdef LINUX
#include <dlfcn.h>
#endif

#include "projectManager.h"
#include "projectManagerWindow.h"
#include "mosaickerWindow.h"
#include "newProjectDialog.h"
#include "newProjectDialogS2.h"
#include "imagePane.h"
#include "content.h"
#include "addCameraDialog.h"
#include "opgWindow.h"
#include "itemInfoPane.h"
#include "imageProcessorWindow.h"
#include "imageFactory.h"

BEGIN_EVENT_TABLE(ProjectManagerWindow, wxWindow)
EVT_MENU(IdLoadProject, ProjectManagerWindow::onOpenProject)
EVT_MENU(IdExit, ProjectManagerWindow::onExit)
EVT_MENU(IdMosaicker, ProjectManagerWindow::onOpenMosaicker)
EVT_MENU(IdOPG, ProjectManagerWindow::onOpenOPG)
EVT_MENU(IdAddCamera, ProjectManagerWindow::onAddCameraToProject)
EVT_MENU(IdCreateProject, ProjectManagerWindow::onNewProject)
EVT_MENU(IdSaveAsProject, ProjectManagerWindow::onSaveAsProject)
EVT_MENU(IdSaveProject, ProjectManagerWindow::onSaveProject)
EVT_MENU(IdCloseProject, ProjectManagerWindow::onCloseProject)
EVT_MENU(IdAddImage, ProjectManagerWindow::onAddImage)
EVT_TREE_BEGIN_LABEL_EDIT(IdProjectTree, ProjectManagerWindow::onProjectTreeLabelEditBegin)
EVT_TREE_END_LABEL_EDIT(IdProjectTree, ProjectManagerWindow::onProjectTreeLabelEditEnd)
EVT_TREE_ITEM_ACTIVATED(IdProjectTree, ProjectManagerWindow::onTreeItemClicked)
EVT_TREE_ITEM_MENU(IdProjectTree, ProjectManagerWindow::onTreeItemMenuRequest)
END_EVENT_TABLE()

///<todo>Loading the last opened project on workspace should be added
///to codebase.</todo>
ProjectManagerWindow::ProjectManagerWindow() :
  wxFrame(NULL, wxID_ANY, _("Photokan - Main View"), wxPoint(DEFAULTXPOS, DEFAULTYPOS), wxSize(DEFAULTWIDTH, DEFAULTHEIGHT))
{
  projectTree = NULL; 
  contentBook = NULL;
  itemInfoPane = NULL;
  nPlayers = 0;

  pm = new ProjectManager();

  layoutMgr.SetManagedWindow(this);
  wxMenuBar* pmMenuBar = new wxMenuBar();
  wxMenu* pmFileMenu = new wxMenu();
  wxMenu* pmProjectMenu = new wxMenu();
  wxMenu* pmToolsMenu = new wxMenu();

  pmFileMenu->Append(IdCreateProject, _("&New Project\tCtrl+N"), _("Create a new project"));
  pmFileMenu->Append(IdSaveProject, _("&Save Project\tCtrl+S"), _("Save current project"));
  pmFileMenu->Append(IdSaveAsProject, _("&Save Project As..\tCtrl+Shift+S"), _("Save current project as.."));
  pmFileMenu->Append(IdLoadProject, _("&Load Project\tCtrl+O"), _("Load a project"));
  pmFileMenu->Append(IdCloseProject, _("&Close Project"), _("Close current project"));
  pmFileMenu->AppendSeparator();
  pmFileMenu->Append(IdExit, _("&Exit\tCtrl+W"), _("Exit Photokan"));

  pmToolsMenu->Append(IdMosaicker, _("&Mosaicker Tool\tCtrl+M"), _("Load the mosaicking tool"));
  pmToolsMenu->Append(IdOPG, _("Orthophoto Generator\tCtrl+P"), _("Load the orthophoto generator tool"));

  pmProjectMenu->Append(IdAddCamera, _("Add New &Camera"), _("Add a new camera to project"));
  pmProjectMenu->Append(IdAddImage, _("Add New &Image"), _("Add a new image to project"));

  pmMenuBar->Append(pmFileMenu, _("File"));
  pmMenuBar->Append(pmProjectMenu, _("Project"));
  pmMenuBar->Append(pmToolsMenu, _("Tools"));
  
  SetMenuBar(pmMenuBar);
  
  //Add the image processor window as a toolbar
  if(loadIP() == 1) {
    layoutMgr.AddPane(ip, wxAuiPaneInfo().Name(_("Image Processor")).Caption(_("Image Processor")).Top().Layer(1).Position(4).Dockable().Floatable().CloseButton().Resizable(false).MinSize(wxSize(75,75)) );
  }
  //associate the popup menu
  if(createProjectTree() == 1) {
    layoutMgr.AddPane(projectTree, wxAuiPaneInfo().Name(_("Project Tree")).Caption(_("Project Tree")).Left().Layer(1).Position(1).Dockable().Floatable().CloseButton().MaximizeButton());
  }
  //content book does not show
  if(createContentBook() == 1) {
    layoutMgr.AddPane(contentBook, wxAuiPaneInfo().Name(_("Content Book")).Caption(_("Content Book")).CenterPane().Layer(1).Position(2).Dockable().Floatable().CloseButton().MaximizeButton());
  }
  if(createItemInfoPane() == 1) {
    layoutMgr.AddPane(itemInfoPane, wxAuiPaneInfo().Name(_("Content Info Pane")).Caption(_("Content Info Pane")).Right().Layer(1).Position(3).Dockable().Floatable().CloseButton().MaximizeButton());
  }

  createPopUpMenus();

  CreateStatusBar();
  layoutMgr.Update();
}

int ProjectManagerWindow::createItemInfoPane() {
  if(itemInfoPane)
    return -1;
  itemInfoPane = new ItemInfoPane(this, IdInfoPane, wxDefaultPosition, wxSize(150, 300));
  return 1;
}
ProjectManagerWindow::~ProjectManagerWindow() { 

}

int ProjectManagerWindow::loadIP() {
  ip = new ImageProcessorWindow(this, this, IdIP);
  return 1;
}

Content* ProjectManagerWindow::getSelectedContent() {
  ImagePanel* pane = (ImagePanel*)contentBook->GetPage(contentBook->GetSelection());
  Image* im = pane->getImage();
  
  return im;
}

ImagePanel* ProjectManagerWindow::getSelectedPane() {
  return (ImagePanel*)contentBook->GetPage(contentBook->GetSelection());
}
///<summary>Loads the mosaicking tool.</summary>
///<todo>Deafult mosaick module kaydetme ozelligi iyi olur. </todo>
int ProjectManagerWindow::loadMosaickerTool() {
  MosaickerWindow* mw = new MosaickerWindow(pm, this, wxID_ANY, _("Mosaicker Tool"), wxDefaultPosition, wxSize(350, 550));
  if(mw->ShowModal() == wxID_OK)
    updateProjectTree();
	return 1;
}

void ProjectManagerWindow::onOpenMosaicker(wxCommandEvent&) {
  if(!pm) {
    wxMessageDialog err(this, _("You should load or create a project first."), _("Error"), wxICON_EXCLAMATION);
    err.ShowModal();
    return;		    
  }
  loadMosaickerTool();
}

///<summary>Opens a new object</summary>
void ProjectManagerWindow::onOpenProject(wxCommandEvent& evt) {
  string projectPath;
  ///<todo> file dialog sacmalio, project filtrelerken</todo>
  wxFileDialog* fileDialog = new wxFileDialog(this, _("Choose a project file to open"), wxEmptyString, wxEmptyString); //, _("Project Files (*.proj)"));

  if(fileDialog->ShowModal() == wxID_OK) {
    projectPath = wxConvCurrent->cWX2MB(fileDialog->GetPath());
    pm->openProject(projectPath);
    updateProjectTree();
  }
}

///<summary> Creates the project tree object</summary>
///<remarks> Initializes the the project tree and associates the
///folder and file icons with the tree</remarks>
///<return> 1 is returned on success otherwise an error code is
///returned. </return>
///<todo> If the last open workspace is loaded during startup the tree
///should be populated with project items.</todo>
int ProjectManagerWindow::createProjectTree() {
  if(projectTree)
    return -1;

  projectTree = new wxTreeCtrl(this, IdProjectTree, wxPoint(0, 0), wxSize(160, 320), wxTR_DEFAULT_STYLE | wxTR_MULTIPLE | wxTR_EDIT_LABELS);
  wxImageList* treeIconList = new wxImageList(16, 16, true, 2);
  treeIconList->Add(wxArtProvider::GetBitmap(wxART_FOLDER, wxART_OTHER, wxSize(16, 16)));
  treeIconList->Add(wxArtProvider::GetBitmap(wxART_NORMAL_FILE, wxART_OTHER, wxSize(16, 16)));
  projectTree->AssignImageList(treeIconList);
  
  projectTree->AddRoot(_("No Project Loaded"), 0);
  return 1;
}
///<summary>Initializes the notebook on which images and other related
///products are shown.
int ProjectManagerWindow::createContentBook() {
  if(contentBook)
    return -1;

  contentBook = new wxAuiNotebook(this, IdContentBook);
  return 1;
}
void ProjectManagerWindow::onExit(wxCommandEvent& evt) { 
#ifdef WIN32
  ExitProcess(1);
#else
  exit(0);
#endif
}
///<summary>The contents of the tree controller are updated according
///to the current contents of the open project</summary>
int ProjectManagerWindow::updateProjectTree() {
  wxTreeItemId projectRoot, demRoot, imagesRoot, orthophotoRoot, mosaicRoot, spRoot, cameraRoot, videosRoot;

  if(!projectTree)
    return -1;

  projectTree->DeleteAllItems();

  if(!pm->projectLoaded()) {
    projectTree->AddRoot(_("No project loaded"), 0);

    return -2;
  }
  ProjectPresenter* pp = pm->getProjectPresenter();
  string projectName = pp->getProjectName();
  TreeContent* newContent;

  projectTree->DeleteAllItems();

  projectRoot = projectTree->AddRoot(wxString(projectName.c_str(), wxConvUTF8).c_str(), 0);
  newContent = new TreeContent(TreeContent::HeaderPRItem, projectName);
  newContent->SetId(projectRoot);
  projectTree->SetItemData(projectRoot, newContent);

  
  imagesRoot = projectTree->AppendItem(projectRoot, _("Images"), 0);
  newContent = new TreeContent(TreeContent::HeaderIRItem);
  newContent->SetId(imagesRoot);
  projectTree->SetItemData(imagesRoot, newContent);

  videosRoot = projectTree->AppendItem(projectRoot, _("Videos"), 0);
  newContent = new TreeContent(TreeContent::HeaderVRItem);
  newContent->SetId(videosRoot);
  projectTree->SetItemData(videosRoot, newContent);

  cameraRoot = projectTree->AppendItem(projectRoot, _("Cameras"), 0);
  newContent = new TreeContent(TreeContent::HeaderCRItem);
  newContent->SetId(cameraRoot);
  projectTree->SetItemData(cameraRoot, newContent);

  demRoot = projectTree->AppendItem(projectRoot, _("Digital Elevation Models"), 0);
  newContent = new TreeContent(TreeContent::HeaderDRItem);
  newContent->SetId(demRoot);
  projectTree->SetItemData(demRoot, newContent);

  orthophotoRoot = projectTree->AppendItem(projectRoot, _("Orthophotos"), 0);
  newContent = new TreeContent(TreeContent::HeaderORItem);
  newContent->SetId(orthophotoRoot);
  projectTree->SetItemData(orthophotoRoot, newContent);

  mosaicRoot = projectTree->AppendItem(projectRoot, _("Mosaics"), 0);
  newContent = new TreeContent(TreeContent::HeaderMRItem);
  newContent->SetId(mosaicRoot);
  projectTree->SetItemData(mosaicRoot, newContent);

  spRoot = projectTree->AppendItem(projectRoot, _("Super Resolution"), 0);
  newContent = new TreeContent(TreeContent::HeaderSRItem);
  newContent->SetId(spRoot);
  projectTree->SetItemData(spRoot, newContent);


  vector<pair <string, ContentType> >* contentList = pp->enumerateContents();

  for(vector<pair<string, ContentType> >::iterator i = contentList->begin(); i != contentList->end(); i++) {
    wxTreeItemId newItem;
    if(i->second == Image2D) {
      newItem = projectTree->AppendItem(imagesRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    if(i->second == VideoType) {
      newItem = projectTree->AppendItem(videosRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    else if(i->second == DEM) {
      newItem = projectTree->AppendItem(demRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    else if(i->second == OrthophotoType) {
      newItem = projectTree->AppendItem(orthophotoRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    else if(i->second == Mosaic) {
      newItem = projectTree->AppendItem(mosaicRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    if(i->second == SuperResolution) {
      newItem = projectTree->AppendItem(spRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }
    if(i->second == CameraType) {
      newItem = projectTree->AppendItem(cameraRoot, wxString(i->first.c_str(), *wxConvCurrent), 1);
    }

    ///<todo> Tree uzerinden direct access icin bir ayar lazim</todo>
    TreeContent* itemData = new TreeContent(TreeContent::FileItem, i->first);
    itemData->SetId(newItem);
    projectTree->SetItemData(newItem, itemData);
		
  }
  return 1;
}

void ProjectManagerWindow::onProjectTreeLabelEditBegin(wxTreeEvent& evt) {
  if(!pm->projectLoaded()) {
    evt.Veto();
    return;
  }
  TreeContent* nodeContent = dynamic_cast<TreeContent*>(projectTree->GetItemData(evt.GetItem()));
  if(nodeContent->getItemType() != TreeContent::HeaderPRItem ||
     nodeContent->getItemType() != TreeContent::FileItem) {
    evt.Veto();
    return;
  }
}

void ProjectManagerWindow::onProjectTreeLabelEditEnd(wxTreeEvent& evt) {
  int result;
  ProjectPresenter* pp = pm->getProjectPresenter();

  wxTreeItemId elem = evt.GetItem(); 
  TreeContent* nodeContent = dynamic_cast<TreeContent*>(projectTree->GetItemData(elem));
  if(nodeContent->getItemType() == TreeContent::HeaderPRItem) {
    string newProjectName = (const char*) wxConvCurrent->cWX2MB(projectTree->GetItemText(elem));
    result = pm->renameProject(newProjectName);
    if(result) {
      projectTree->SetItemText(elem, wxString(pp->getProjectName().c_str(), wxConvUTF8));
    }
  }
  else
    evt.Veto();
}

void ProjectManagerWindow::onNewProject(wxCommandEvent& evt) {
  NewProjectDialog* npd = new NewProjectDialog(this, wxID_ANY);
  NewProjectDialogS2* npd2 = new NewProjectDialogS2(this, wxID_ANY);
  string projectName;
  string projectAddress;

  if(npd->ShowModal() == wxID_OK) {
    if(npd2->ShowModal() == wxID_OK) {
      projectName = (const char* )wxConvCurrent->cWX2MB(npd->projectName);
      projectAddress = (const char* )wxConvCurrent->cWX2MB(npd->projectPath);

      vector<string> pathList = npd2->getImageList();

      pm->newProject(projectName, pathList, true, projectAddress, true);
      updateProjectTree();

    }
  }
  
} 

void ProjectManagerWindow::onSaveAsProject(wxCommandEvent& evt) {
  wxFileDialog* saveAsDialog = new wxFileDialog(this, _("Choose a project file name to save"), wxEmptyString, wxEmptyString);
  if(saveAsDialog->ShowModal() == wxID_OK) {
    pm->saveAsProject((const char*)wxConvCurrent->cWX2MB(saveAsDialog->GetPath()));
  }
}

void ProjectManagerWindow::onTreeItemClicked(wxTreeEvent& evt) {
  TreeContent* itemContent;
  wxTreeItemId it = evt.GetItem();
  itemContent = dynamic_cast<TreeContent*>(projectTree->GetItemData(it));

  if(!itemContent)
    return;

  Content* content;
  ProjectPresenter* pp = pm->getProjectPresenter();
  map<string, Content*>* contentMap = pp->getContentsByName();
  
  if(itemContent->getItemType() == TreeContent::HeaderPRItem) {
    itemInfoPane->setContent(pp);
    itemInfoPane->update();
    return;
  }
  if(itemContent->getItemType() == TreeContent::HeaderIRItem ||
     itemContent->getItemType() == TreeContent::HeaderDRItem ||
     itemContent->getItemType() == TreeContent::HeaderORItem ||
     itemContent->getItemType() == TreeContent::HeaderSRItem ||
     itemContent->getItemType() == TreeContent::HeaderMRItem ||
     itemContent->getItemType() == TreeContent::HeaderCRItem) {
    return;
  }    

  content = (*contentMap->find(itemContent->getItemId())).second;
  
  itemInfoPane->setContent(content);
  itemInfoPane->update();
  if(content->contentType == Image2D || content->contentType == OrthophotoType || content->contentType == Mosaic || content->contentType == DEM) {

    wxRect pageRect = contentBook->GetClientRect();
    int width, height;
    
    width = pageRect.GetWidth();
    height = pageRect.GetHeight();

    wxSize nsize = contentBook->GetClientSize();

    Image* image = static_cast<Image*>(content);
    ImagePanel* panel = new ImagePanel(image, contentBook, wxID_ANY, wxPoint(0, 0), nsize, wxTAB_TRAVERSAL | wxBORDER_DEFAULT, true, false);

    contentBook->SetAutoLayout(false);

    panel->Connect(wxEVT_MOTION, wxMouseEventHandler(ProjectManagerWindow::onMouseMoveOverPane), NULL, this);
    contentBook->AddPage(panel, wxString(image->name.c_str(),wxConvUTF8));
    contentBook->SetSelection(contentBook->GetPageCount() - 1);

    ip->setSliders(0);
    SetStatusText(_(""));
    contentBook->Refresh();

    panel->Refresh();

    Update();
    return;
  } 
  if(content->contentType == VideoType) {
    wxSize nsize = contentBook->GetClientSize();
    Video* video = (Video*)content;

    Image* image = ImageFactory::fromIplHandle(video->grabFrame(30), "Sample Frame");
    ImagePanel* panel = new ImagePanel(image, contentBook, wxID_ANY, wxPoint(0, 0), nsize, wxTAB_TRAVERSAL | wxBORDER_DEFAULT, true, false);

    contentBook->SetAutoLayout(false);

    panel->Connect(wxEVT_MOTION, wxMouseEventHandler(ProjectManagerWindow::onMouseMoveOverPane), NULL, this);
    contentBook->AddPage(panel, wxString(image->name.c_str(),wxConvUTF8));
    contentBook->SetSelection(contentBook->GetPageCount() - 1);

    ip->setSliders(0);
    SetStatusText(_(""));
    contentBook->Refresh();

    panel->Refresh();

    Update();
    return;    
  }
}

void ProjectManagerWindow::onOpenOPG(wxCommandEvent& evt) {
  if(!pm) {
    wxMessageDialog err(this, _("You should load or create a project first."), _("Error"), wxICON_EXCLAMATION);
    err.ShowModal();
    return;
  }
  OPGWindow* opgw = new OPGWindow(pm, this, wxID_ANY);
  if(opgw->ShowModal() == wxID_OK)
    updateProjectTree();


}

void ProjectManagerWindow::onAddCameraToProject(wxCommandEvent& evt) {
  AddCameraDialog* acd = new AddCameraDialog(this, wxID_ANY);
  Camera* newCamera;
  if(acd->ShowModal() == wxID_OK) {
    newCamera = new Camera(acd->name, "");
    newCamera->f = acd->f;
    newCamera->x = acd->x;
    newCamera->y = acd->y;
    newCamera->z = acd->z;
    newCamera->px = acd->px;
    newCamera->py = acd->py;
    newCamera->pSize = acd->pSize;
    newCamera->omega = acd->omega;
    newCamera->phi = acd->phi;
    newCamera->kappa = acd->kappa;

    pm->addCameraToProject(newCamera);
    updateProjectTree();
  }
}

void ProjectManagerWindow::onSaveProject(wxCommandEvent& evt) {
  pm->saveProject();
}

void ProjectManagerWindow::onCloseProject(wxCommandEvent& evt) {
  bool saveFirst = false;
  if(pm->projectModified()) {
    wxMessageDialog prompt(this, _("Would you like to save the project?"), _("Close Project"), wxYES_NO);
    if(prompt.ShowModal() == wxID_YES) {
      saveFirst = true;
    }
  }
  pm->closeProject(saveFirst);
  itemInfoPane->setContent((Content*)NULL);
  itemInfoPane->setContent((ProjectPresenter*)NULL);
  itemInfoPane->update();
  ip->setSliders(0);
  updateProjectTree();
  closePanes();
  Refresh();
}

int ProjectManagerWindow::closePanes() {
  unsigned int pageCount = contentBook->GetPageCount();
  for(unsigned int i = 0; i < pageCount; i++) {
    contentBook->DeletePage(0);
  }
  return 1;
}

void ProjectManagerWindow::onMouseMoveOverPane(wxMouseEvent& evt) {
  int xUnit, yUnit, xStart, yStart, x, y;
  ImagePanel* currentPane = getSelectedPane();
  Content* content = getSelectedContent();
  if(content->contentType != Image2D &&
     content->contentType != OrthophotoType &&
     content->contentType != Mosaic) {
    return;
  }
  Image* selImage = static_cast<Image*>(content);

  double* worldData = selImage->getWorldData();
  if(!worldData) {
    return;
  }
  currentPane->GetScrollPixelsPerUnit(&xUnit, &yUnit);
  currentPane->GetViewStart(&xStart, &yStart);
  x = evt.GetX() + xUnit * xStart;
  y = evt.GetY() + yUnit * yStart;

  double xCoordinate = x * worldData[1] + y * worldData[4] + worldData[0];
  double yCoordinate = x * worldData[2] + y * worldData[5] + worldData[3];

  SetStatusText(wxString::Format(_("X: %0.5f Y: %0.5f"), xCoordinate, yCoordinate));
}
///WIN32 case handle edilmedi!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
void ProjectManagerWindow::onAddImage(wxCommandEvent& evt) {
  wxFileDialog* fDialog = new wxFileDialog(this, _("Please choose the image you want to add to the project"), wxEmptyString, wxEmptyString);
  if(fDialog->ShowModal() == wxID_OK) {
    string filePath = (const char*)wxConvCurrent->cWX2MB(fDialog->GetPath());
    string fileName = (const char*)wxConvCurrent->cWX2MB(fDialog->GetFilename());
    ProjectPresenter* pp = pm->getProjectPresenter();
#ifdef LINUX
    string cpcom = "cp " + filePath + " " + pp->getProjectAddress();
    system(cpcom.c_str());
    pm->addImageToProject(fileName, pp->getProjectAddress() + "/" + fileName, "Imported");
#elif WIN32
#endif
    updateProjectTree();
  }
  else {
    return;
  }
}

int ProjectManagerWindow::createPopUpMenus() {
  treeMenu = new wxMenu(_("Edit"));
  cameraList = new wxMenu(_("Cameras"));
  demList = new wxMenu(_("DEMs"));
  treeMenu->AppendSubMenu(cameraList, _("Associate Camera"));
  treeMenu->AppendSubMenu(demList, _("Associate DEM"));

  return 1;
}

void ProjectManagerWindow::onTreeItemMenuRequest(wxTreeEvent& evt) {

  for(unsigned int i = 0; i < cameraList->GetMenuItemCount(); i++) {
    cameraList->Delete(IdCameraList + i);
  }

  for(unsigned int i = 0; i < demList->GetMenuItemCount(); i++) {
    demList->Delete(IdDemList + i);
  }
  
  ProjectPresenter* pp = pm->getProjectPresenter();
  map<string, Content*>* contents = pp->getContentsByName();
  string cameraName;
  string demName;
  unsigned int demCount = 0;
  unsigned int cameraCount = 0;

  for(map<string, Content*>::iterator i = contents->begin() ; i != contents->end(); i++) {
    Content* content = i->second;
    Image* selImage;
    wxTreeItemId itemId = evt.GetItem();
    string itemName = (const char*)wxConvCurrent->cWX2MB(projectTree->GetItemText(itemId));

    if(content->contentType == Image2D && content->name == itemName) {
      selImage = (Image*)content;
      cameraName = selImage->getCamera();
      demName = selImage->getDem();
    }
  }
  int count = 0;
  for(map<string, Content*>::iterator i = contents->begin() ; i != contents->end(); i++, count++) {
    ContentType content = i->second->contentType;
    string contentName =  i->first;
    if(content == CameraType) {
      cameraList->AppendCheckItem(IdCameraList + cameraCount, wxString(contentName.c_str(), wxConvUTF8));
      if(contentName == cameraName) {
	cameraList->Check(IdCameraList + cameraCount, true);      
      }
      cameraCount++;
    }

    if(content == DEM) {
      demList->AppendCheckItem(IdDemList + demCount, wxString(contentName.c_str(), wxConvUTF8));
      if(contentName == demName) {
	demList->Check(IdDemList + demCount, true);
      }
      demCount++;
    }
  }
  Connect(IdCameraList, IdCameraList + cameraCount, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(ProjectManagerWindow::onSetCamera));
  Connect(IdDemList, IdDemList + demCount, wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(ProjectManagerWindow::onSetDem));

  projectTree->PopupMenu(treeMenu, evt.GetPoint());
}

void ProjectManagerWindow::onSetCamera(wxCommandEvent& evt) {
  string cameraName;
  string fileName;
  
  cameraName = (const char*)wxConvCurrent->cWX2MB(cameraList->GetLabel(evt.GetId()));
  fileName = (const char*)wxConvCurrent->cWX2MB(projectTree->GetItemText(projectTree->GetSelection()));

  ProjectPresenter* pp = pm->getProjectPresenter();
  map<string, Content*>* contents = pp->getContentsByName();

  for(map<string, Content*>::iterator i = contents->begin(); i != contents->end(); i++) {
    Content* content = i->second;
    Image* selImage = (Image*)content;
    if(fileName == i->first) {
      selImage->setCamera(cameraName);
      break;
    }
  }
  
}

void ProjectManagerWindow::onSetDem(wxCommandEvent& evt) {

}

void ProjectManagerWindow::onMediaLoaded(wxMediaEvent& evt) {
  ((wxMediaCtrl*)FindWindow(evt.GetId()))->Play();
}
