#ifndef __PROJECTMANAGERWINDOW_H__
#define __PROJECTMANAGERWINDOW_H__

#include <wx/wx.h>
#include <wx/treectrl.h>
#include <wx/aui/aui.h>
#include <wx/mediactrl.h>
#include "projectManager.h"
#include "itemInfoPane.h"
#include "imageProcessorWindow.h"
#include "imageProcessor.h"
#include "imagePane.h"

#define DEFAULTXPOS 50
#define DEFAULTYPOS 50
#define DEFAULTWIDTH 800
#define DEFAULTHEIGHT 600

///<summary>ProjectManagerWindow is the gui class of Photokan's
///project manager.</summary>
class ProjectManagerWindow : public wxFrame {
  friend class ImageProcessorWindow;
 protected:
  DECLARE_EVENT_TABLE()
 private:
  ///<summary>pm variable realizes the project management facilities
  ///of Photokan.</summary>
  ProjectManager* pm;
  ///<summary> The ids of elements of the project manager gui</summary>
  enum {
    IdOpenProject = wxID_HIGHEST + 1,
    IdCreateProject,
    IdSaveProject,
    IdSaveAsProject,
    IdLoadProject,
    IdCloseProject,
    IdExit,
    IdProjectTree,
    IdContentBook,
    IdMosaicker,
    IdOPG,
    IdAddCamera,
    IdAddImage,
    IdInfoPane,
    IdIP,
    IdCameraList,
    IdDemList = IdCameraList + 100,
    IdVideoPlayer = IdDemList + 100
  };
  wxAuiManager layoutMgr;
  wxAuiNotebook* contentBook;
  wxTreeCtrl* projectTree;
  ItemInfoPane* itemInfoPane;
  ImageProcessorWindow* ip;
  wxMenu* treeMenu;
  wxMenu* demList;
  wxMenu* cameraList;
  int nPlayers;
 private:
  int createProjectTree();
  int createContentBook();
  int createItemInfoPane();
  int createPopUpMenus();
  int loadMosaickerTool();
  int updateProjectTree();
  int loadIP();
  int closePanes();

  void onOpenMosaicker(wxCommandEvent&);
  void onOpenOPG(wxCommandEvent&);
  void onOpenProject(wxCommandEvent&);
  void onAddCameraToProject(wxCommandEvent&);
  void onExit(wxCommandEvent&);
  void onProjectTreeLabelEditBegin(wxTreeEvent&);
  void onProjectTreeLabelEditEnd(wxTreeEvent&);
  void onNewProject(wxCommandEvent&);
  void onSaveAsProject(wxCommandEvent&);
  void onSaveProject(wxCommandEvent&);
  void onCloseProject(wxCommandEvent&);
  void onTreeItemClicked(wxTreeEvent&);
  void onMouseMoveOverPane(wxMouseEvent&);
  void onAddImage(wxCommandEvent&);
  void onTreeItemMenuRequest(wxTreeEvent&);
  void onSetCamera(wxCommandEvent&);
  void onSetDem(wxCommandEvent&);
  void onMediaLoaded(wxMediaEvent&);
 public:
  ///<summary>ProjectManagerWindow constructor.</summary>
  ///<remarks> All necessary values like window title, style etc. are
  ///hardwired into the definition of the method.</remarks>
  ProjectManagerWindow();
  virtual ~ProjectManagerWindow();  
  Content* getSelectedContent();
  ImagePanel* getSelectedPane();
};

class TreeContent : public wxTreeItemData {
 public:
  enum TreeItemType {
    HeaderIRItem,
    HeaderVRItem,
    HeaderDRItem,
    HeaderORItem,
    HeaderSRItem,
    HeaderMRItem,
    HeaderPRItem,
    HeaderCRItem,
    FileItem
  };
 private:
  TreeItemType itemType;
  string itemId;
 public:
  TreeContent(TreeItemType itemType, string itemId = "") {
    this->itemType = itemType;
    this->itemId = itemId;
  }

  TreeItemType getItemType() {
    return itemType;
  }

  string getItemId() {
    return itemId;
  }
};
#endif
